from opendatasets.utils import download_url

URLs = [
    'https://raw.githubusercontent.com/owid/covid-19-data/master/public/data/owid-covid-data.csv',
    'https://raw.githubusercontent.com/owid/covid-19-data/master/public/data/owid-covid-data-last-updated-timestamp.txt',
    'https://raw.githubusercontent.com/owid/covid-19-data/master/public/data/owid-covid-codebook.csv'
]


def download(dataset_id, data_dir, dry_run, **kwargs):
    for url in URLs:
        download_url(url, data_dir, dry_run=dry_run)



from opendatasets.utils.network import get_filename_cd
import os
import re
import zipfile
import cgi  
from opendatasets.utils.md5 import check_integrity
from urllib.parse import urlparse
from tqdm import tqdm


def download_google_drive(url, data_dir):
    print('Downloading from Google Drive (may take a while):', url)
    file_id = _get_google_drive_file_id(url)
    fpath = download_file_from_google_drive(file_id, data_dir)
    if fpath and str(fpath).endswith('.zip'):
        try:
            with zipfile.ZipFile(fpath) as z:
                z.extractall(fpath[:-4])
            print('Downloaded and unzipped to ', fpath[:-4])
        except zipfile.BadZipFile as e:
            raise ValueError('Bad zip file! Please retry', e)
        try:
            os.remove(fpath)
        except OSError as e:
            print('Could not delete zip file, got %s' % e)
    else:
        print('Downloaded to ', fpath)


def is_google_drive_url(url):
    return url.startswith('https://drive.google.com') or url.startswith('http://drive.google.com') or url.startswith('drive.google.com')


def _get_google_drive_file_id(url):
    parts = urlparse(url)

    if re.match(r"(drive|docs)[.]google[.]com", parts.netloc) is None:
        return None

    match = re.match(r"/file/d/(?P<id>[^/]*)", parts.path)
    if match is None:
        return None

    return match.group("id")

def _quota_exceeded(response):
    return "Google Drive - Quota exceeded" in response.text


def download_file_from_google_drive(file_id, root, filename=None, md5=None):
    """Download a Google Drive file from  and place it in root.
    Args:
        file_id (str): id of file to be downloaded
        root (str): Directory to place downloaded file in
        filename (str, optional): Name to save the file under. If None, use the id of the file.
        md5 (str, optional): MD5 checksum of the download. If None, do not check
    """
    # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
    try:
        import requests
    except ImportError as e:
        print(
            'To download from Google Drive, first install the requests library by running:')
        print('pip install requests --upgrade')
        raise e
    url = "https://docs.google.com/uc?export=download"

    root = os.path.expanduser(root)
    fpath = os.path.join(root, filename or file_id)

    os.makedirs(root, exist_ok=True)

    if os.path.isfile(fpath) and check_integrity(fpath, md5):
        print('Using downloaded and verified file: ' + fpath)
    else:
        session = requests.Session()
        response = session.get(url, params={'id': file_id}, stream=True)
        token = _get_confirm_token(response)
        if token:
            params = {'id': file_id, 'confirm': token}
            response = session.get(url, params=params, stream=True)

        if not filename:
            filename = get_filename_cd(response)
        if not filename:
            filename = file_id
        fpath = os.path.join(root, filename)

        if _quota_exceeded(response):
            msg = (
                "The daily quota of the file " + filename + " is exceeded and it " +
                "can't be downloaded. This is a limitation of Google Drive " +
                "and can only be overcome by trying again later."
            )
            raise RuntimeError(msg)

        _save_response_content(response, fpath)
        return fpath


def _get_confirm_token(response):
    for key, value in response.cookies.items():
        if key.startswith('download_warning'):
            return value

    return None


def _save_response_content(response, destination, chunk_size=32768):
    with open(destination, "wb") as f:
        pbar = tqdm(total=None)
        progress = 0
        for chunk in response.iter_content(chunk_size):
            if chunk:  # filter out keep-alive new chunks
                f.write(chunk)
                progress += len(chunk)
                pbar.update(progress - pbar.n)
        pbar.close()



import os
import hashlib


def calculate_md5(fpath, chunk_size=1024 * 1024):
    md5 = hashlib.md5()
    with open(fpath, 'rb') as f:
        for chunk in iter(lambda: f.read(chunk_size), b''):
            md5.update(chunk)
    return md5.hexdigest()


def check_md5(fpath, md5, **kwargs):
    return md5 == calculate_md5(fpath, **kwargs)


def check_integrity(fpath, md5=None):
    if not os.path.isfile(fpath):
        return False
    if md5 is None:
        return True
    return check_md5(fpath, md5)



import importlib
from opendatasets.utils.network import download_url, is_url
from opendatasets.utils.googledrive import is_google_drive_url, download_google_drive
import os
from opendatasets._version import __version__
from opendatasets.utils.kaggle_api import download_kaggle_dataset, is_kaggle_url
from opendatasets.utils.archive import extract_archive


def download(dataset_id_or_url, data_dir='.', force=False, dry_run=False, **kwargs):
    # Check for a Kaggle dataset URL
    if is_kaggle_url(dataset_id_or_url):
        return download_kaggle_dataset(dataset_id_or_url, data_dir=data_dir, force=force, dry_run=dry_run)

    # Check for Google Drive URL
    if is_google_drive_url(dataset_id_or_url):
        return download_google_drive(dataset_id_or_url, data_dir)

    # Download a raw URL
    if is_url(dataset_id_or_url):
        return download_url(dataset_id_or_url, data_dir)

    dataset_id = dataset_id_or_url
    data_dir = os.path.join(data_dir, dataset_id_or_url)
    if not os.path.exists(data_dir):
        os.makedirs(data_dir)
    dataset = importlib.import_module('opendatasets.datasets.' + dataset_id)
    if dry_run:
        print('This is a dry run. URLs will be displayed but the files will not be downloaded.')
    return dataset.download(dataset_id=dataset_id, data_dir=data_dir, dry_run=dry_run, **kwargs)


def version():
    return __version__



from opendatasets.utils.network import download_url

GITHUB_RAW_BASE_URL = 'https://raw.githubusercontent.com/JovianML/opendatasets/master/data/'


def download_raw_files(dataset_id, data_dir, dry_run, files):
    base_url = GITHUB_RAW_BASE_URL + dataset_id + "/"
    for fname in files:
        full_url = base_url + fname
        download_url(full_url, data_dir, fname, dry_run=dry_run)



import os
from opendatasets.utils.kaggle_direct import get_kaggle_dataset_id, is_kaggle_url
from opendatasets.utils.archive import extract_archive
import click
import json


def _get_kaggle_key():
    user_input = click.prompt("Your Kaggle Key", hide_input=True)
    if user_input.startswith('{'):
        try:
            import json
            api_details = json.loads(user_input)
            return api_details['key']
        except Exception:
            return user_input
    return user_input


def read_kaggle_creds():
    try:
        if os.path.exists('./kaggle.json'):
            with open('./kaggle.json', 'r') as f:
                key = f.read()
                data = json.loads(key)
                if 'username' in data and 'key' in data:
                    os.environ['KAGGLE_USERNAME'] = data['username']
                    os.environ['KAGGLE_KEY'] = data['key']
                    return True
    except Exception:
        return False


def download_kaggle_dataset(dataset_url, data_dir, force=False, dry_run=False):
    dataset_id = get_kaggle_dataset_id(dataset_url)
    id = dataset_id.split('/')[1] 
    target_dir = os.path.join(data_dir, id)

    if not read_kaggle_creds():
        print("Please provide your Kaggle credentials to download this dataset. Learn more: http://bit.ly/kaggle-creds")
        os.environ['KAGGLE_USERNAME'] = click.prompt("Your Kaggle username")
        os.environ['KAGGLE_KEY'] = _get_kaggle_key()

    if not force and os.path.exists(target_dir) and len(os.listdir(target_dir)) > 0:
        print('Skipping, found downloaded files in "{}" (use force=True to force download)'.format(target_dir))
        return 
    
    if not dry_run:
        from kaggle import api
        api.authenticate()
        if dataset_id.split('/')[0] == 'c':
            api.competition_download_files(
                id,
                target_dir,
                force=force,
                quiet=False)
            zip_fname = target_dir + '/' + id + '.zip'
            extract_archive(zip_fname, target_dir)
            try:
                os.remove(zip_fname)
            except OSError as e:
                print('Could not delete zip file, got' + str(e))
        else:
            api.dataset_download_files(
                id,
                target_dir,
                force=force,
                quiet=False,
                unzip=True)
        
    else:
        print("This is a dry run, skipping..")



import setuptools
import re

VERSIONFILE = "./opendatasets/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)

if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

with open("README.md", "rb") as fh:
    long_description = fh.read().decode('utf-8', errors='ignore')

setuptools.setup(
    name="opendatasets",
    version=verstr,
    author="Jovian.ml",
    author_email="hello@jovian.ml",
    description="A curated collection of datasets for data analysis & machine learning, downloadable with a single Python command",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JovianML/opendatasets",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires=['tqdm', 'kaggle', 'click']
)
